/*
 * DHT11.h
 *
 * Created: 27.05.2016 08:55:28
 *  Author: steffen.thun
 */ 

#define F_CPU 8e6

#include <util/delay.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "float_to_char.h"

#ifndef DHT11_H_
#define DHT11_H_

#define UART 0
#define LCD 1

#define DDR_DHT11			DDRB
#define PORT_DHT11			PORTB
#define PIN_DHT11			PINB
#define DHT11				PB0// Pullup 4,7k

#define DHT11_out_low		DDR_DHT11 |= (1 << DHT11)
#define DHT11_in			DDR_DHT11 &= ~(1 << DHT11)	// Hi ber Pullup
#define DHT11_is_hi			PIN_DHT11 & (1 << DHT11)
#define DHT11_is_low		!(PIN_DHT11 & (1 << DHT11))

#define ERROR1	"Bussignal LOW"
#define	ERROR2	"No Response High-Signal from Sensor"
#define ERROR3	"No Response Low-Signal from Sensor"
#define ERROR4	"No Preparing for Data Sending"
#define ERROR5	"No Startbit detected"
#define ERROR6	"No Data Bit detected"
#define ERROR7	"Checksum doesn't fit"

/*Discription: Liest die Daten vom Sensor
//Return:	Fehler 1:	Wenn Bussignal dauerhaft auf 0 ist
//			Fehler 2:	Wenn das Bussignal lnger als 255 s vom C auf 'LOW' gehalten wird
//			Fehler 3:	Wenn das Bussignal lnger als 255 s vom Sensor auf 'LOW' gehalten wird
//			Fehler 4:	Wenn das Bussignal lnger als 255 s vom Sensor auf 'HIGH' gehalten wird
//			Fehler 5:	Wenn das Bussignal lnger als 255 s vom Sensor auf 'LOW' gehalten wird, fr ein Bit
//			Fehler 6:	Wenn das Bussignal lnger als 255 s vom Sensor auf 'HIGH' gehalten wird, fr ein Bit
//			General Timeout: 255 ms
*/

/* ------------------------------------- Timing Diagramm ------------------------------------------>
						 20-40 s					    80 s					  26-28s --> 1
						<-- HOST-->					 <-- DHT -->				  <-- DHT -->
VCC --\				   /-----------\				/------------\				 /-----------\				 /
	   \			  /	wait for DHT\ DHT Response / DHT Preparing\			    /  			  \				/
GND		\____________/   Response	 \____________/	 to Send Data  \___________/			   \___________/
		 <-- Host -->				  <-- DHT -->					<-- DHT -->					<-- DHT -->
		 min. 18ms					   min. 80s					 50s --> 1					26-28s --> 1

1.		Bit Host (18ms)
2.		Bit DHT  (20-40s)
3.		Bit Host (80s)
4.		Bit DHT  (80s)
5.		Bit DHT  (50s) (Startbit)
6.		Bit DHT  (70s) --> 1; (26-28s) --> 0
7.		Bit DHT (50s) (Wartezeit)
8.		Bit DHT  (70s) --> 1; (26-28s) --> 0
9.		Bit DHT (50s) (Wartezeit)
10 - 40	Bit DHT  (70s) --> 1; (26-28s) --> 0
*/

void dht11_init(void);
uint8_t dht11(unsigned long *feuchte, unsigned long *temperatur, unsigned char *checksum);
void dht11_output(unsigned long feuchte, unsigned long temperatur,unsigned char checksum, uint8_t error);
void dht11_logik_analyse (void);

#endif /* DHT11_H_ */
